/// <reference lib="dom" />
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @package @wsdot/layer-metadata-soe-client
 */
/** @ignore */
const validLayersUrl = "exts/LayerMetadata/validLayers?f=json";
/** @ignore */
const layerSourcesUrl = "exts/LayerMetadata/layerSources?f=json";
/** @ignore */
const metadataUrl = "exts/LayerMetadata/metadata";
/**
 * An error returned from an ArcGIS service.
 * @example
 * // Inside async function
 * const response = await fetch("https://www.example.com/ArcGIS/rest/services/MyService/query?invalidOption=thisisinvalid");
 * const obj = await response.json();
 * if (obj.error) {
 *  throw new ArcGisError(obj.error);
 * }
 */
export class ArcGisError extends Error {
    /**
     * Creates a new instance of ArcGisError.
     * @param errorInfo - The "error" property of the response object returned from a failed ArcGIS Server request.
     */
    constructor(errorInfo) {
        super(errorInfo.message);
        this.errorInfo = errorInfo;
        this.code = errorInfo.code;
    }
}
/**
 * Gets map service info
 * @param serviceUrl map service URL.
 * @throws {ArcGisError}
 */
export function getServiceInfo(serviceUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield fetch([serviceUrl, "f=json"].join("?"));
        const serviceInfo = (yield response.json());
        if (serviceInfo.error) {
            throw new ArcGisError(serviceInfo.error);
        }
        return serviceInfo;
    });
}
/**
 * Detects if a map service supports the "LayerMetadata" capability.
 * @param serviceUrl Map or Feature service URL.
 * @throws {ArcGisError}
 */
export function detectLayerMetadataSupport(serviceUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const serviceInfo = yield getServiceInfo(serviceUrl);
        if (!serviceInfo.supportedExtensions) {
            return false;
        }
        return /LayerMetadata/.test(serviceInfo.supportedExtensions);
    });
}
/**
 * Gets a list of data sources
 * @param serviceUrl Map service URL
 * @returns An object keyed by data set names with values that are layer ID integers associated with those datasets.
 */
export function getLayerSources(serviceUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = [serviceUrl, layerSourcesUrl].join("/");
        const response = yield fetch(url);
        const layerSources = (yield response.json());
        return layerSources;
    });
}
/**
 * Gets URLs to unique metadata items.
 * @param url map service URL
 * @param layerSources If you have already called getLayerSources, you can pass in the results here to avoid making a duplicate request.
 * @returns Key value pairs. Keys are table names and values are metadata URLs.
 */
export function getMetadataLinks(url, layerSources) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!layerSources) {
            layerSources = yield getLayerSources(url);
        }
        const requestUrl = [url, metadataUrl].join("/");
        const output = {};
        // tslint:disable-next-line:forin
        for (const key in layerSources) {
            output[key] = [requestUrl, layerSources[key][0]].join("/");
        }
        return output;
    });
}
/**
 * Gets a list of valid layer IDs
 * @param url Map service URL
 * @throws {ArcGisError} Thrown if the map service returns an error.
 * @throws {TypeError} Thrown if HTTP request returns an object in unexpected format.
 */
export function getValidLayers(url) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestUrl = [url, validLayersUrl].join("/");
        const response = yield fetch(requestUrl);
        const obj = yield response.json();
        if (obj.layerIds) {
            return obj.layerIds;
        }
        else if (obj.error) {
            throw new ArcGisError(obj.error);
        }
        else {
            throw new TypeError(`response was not expected type: ${JSON.stringify(obj)}`);
        }
    });
}
